@@ -44,12 +44,8 @@ def get_status_name(pair, collection):
 
 def load_config(fname, pair_options=('collections', 'conflict_resolution')):
     c = RawConfigParser()
-    try:
-        with open(fname) as f:
-            c.readfp(f)
-    except IOError:
-        cli_logger.exception('Error while trying to read config file.')
-        sys.exit(1)
+    with open(fname) as f:
+        c.readfp(f)
 
     get_options = lambda s: dict(parse_options(c.items(s), section=s))
 
@@ -57,24 +53,31 @@ def load_config(fname, pair_options=('collections', 'conflict_resolution')):
     pairs = {}
     storages = {}
 
-    def handle_pair(section):
-        pair_name = section[len('pair '):]
-        options = get_options(section)
+    def handle_storage(storage_name, options):
+        storages.setdefault(storage_name, {}).update(options)
+
+    def handle_pair(pair_name, options):
         a, b = options.pop('a'), options.pop('b')
         p, s = split_dict(options, lambda x: x in pair_options)
         pairs[pair_name] = a, b, p, s
 
+    def bad_section(name, options):
+        cli_logger.error('Unknown section: {}'.format(name))
+
+    handlers = {'storage': handle_storage, 'pair': handle_pair}
+
     for section in c.sections():
-        if section.startswith('storage '):
-            name = section[len('storage '):]
-            storages.setdefault(name, {}).update(get_options(section))
-        elif section.startswith('pair '):
-            handle_pair(section)
-        elif section == 'general':
-            general = get_options(section)
+        if ' ' in section:
+            section_type, name = section.split(' ', 1)
         else:
-            cli_logger.error('Unknown section in {}: {}'
-                             .format(fname, section))
+            section_type = name = section
+
+        if section_type == 'general':
+            if general is not None:
+                raise CliError('More than one general section in config file.')
+            general = get_options(section_type)
+        else:
+            handlers.get(section_type, bad_section)(name, get_options(section))
 
     if general is None:
         raise CliError(
@@ -103,7 +106,7 @@ def save_status(path, status_name, status):
                        'user. If you think this is an error, please file '
                        'a bug at {}'.format(base_path, PROJECT_HOME))
     if not os.path.exists(base_path):
-        os.makedirs(base_path)
+        os.makedirs(base_path, 0o750)
 
     with open(full_path, 'w+') as f:
         for k, v in status.items():
@@ -137,9 +140,6 @@ def storage_instance_from_config(config, description=None):
         missing = required - given
         invalid = given - all
 
-        cli_logger.critical('error: Failed to initialize {}'
-                            .format(description or cls.storage_name))
-
         if not missing and not invalid:
             cli_logger.exception('')
 
@@ -153,10 +153,25 @@ def storage_instance_from_config(config, description=None):
                 u'error: {} storage doesn\'t take the parameters: {}'
                 .format(cls.storage_name, u', '.join(invalid)))
 
-        sys.exit(1)
+        raise CliError('error: Failed to initialize {}'
+                       .format(description or cls.storage_name))
 
 
 def expand_collection(pair, collection, all_pairs, all_storages):
+    '''
+    Replace the placeholder collections "from a" and "from b" with actual
+    ones.
+
+    :param collection: The collection.
+    :param pair: The pair the collection belongs to.
+    :param all_pairs: dictionary: pair_name => (name of storage a,
+                                                name of storage b,
+                                                pair config,
+                                                storage defaults)
+    :returns: One or more collections that replace the given one. The original
+        collection is returned unmodified if the given collection is neither
+        "from a" nor "from b".
+    '''
     if collection in ('from a', 'from b'):
         a_name, b_name, _, storage_defaults = all_pairs[pair]
         config = dict(storage_defaults)
@@ -171,6 +186,10 @@ def expand_collection(pair, collection, all_pairs, all_storages):
 
 
 def parse_pairs_args(pairs_args, all_pairs):
+    '''
+    Expand the various CLI shortforms ("pair, pair/collection") to an iterable
+    of (pair, collection).
+    '''
     if not pairs_args:
         pairs_args = list(all_pairs)
     for pair_and_collection in pairs_args:
@@ -182,10 +201,9 @@ def parse_pairs_args(pairs_args, all_pairs):
             a_name, b_name, pair_options, storage_defaults = \
                 all_pairs[pair]
         except KeyError:
-            cli_logger.critical('Pair not found: {}'.format(pair))
-            cli_logger.critical('These are the pairs found: ')
-            cli_logger.critical(list(all_pairs))
-            sys.exit(1)
+            raise CliError('Pair not found: {}\n'
+                           'These are the pairs found: {}'
+                           .format(pair, list(all_pairs)))
 
         if collection is None:
             collections = [x.strip() or None for x in
@@ -196,6 +214,9 @@ def parse_pairs_args(pairs_args, all_pairs):
         for c in collections:
             yield pair, c
 
+# We create the app inside a factory and destroy that factory after first use
+# to avoid pollution of the module namespace.
+
 
 def _create_app():
     def catch_errors(f):
@@ -236,7 +257,11 @@ def _create_app():
         if 'config' not in ctx.obj:
             fname = expand_path(os.environ.get('VDIRSYNCER_CONFIG',
                                                '~/.vdirsyncer/config'))
-            ctx.obj['config'] = load_config(fname)
+            try:
+                ctx.obj['config'] = load_config(fname)
+            except Exception as e:
+                raise CliError('Error during reading config{}:\n{}'
+                               .format(fname, e))
 
     @app.command()
     @click.argument('pairs', nargs=-1)
