@@ -61,8 +61,8 @@ class Discover(object):
         :returns: a list of the user's collections (as urls)
         :rtype: list(unicode)
         """
-        for principal in self._find_principal():
-            for home in self._find_home(principal):
+        for principal in list(self._find_principal()) or ['']:
+            for home in list(self._find_home(principal)) or ['']:
                 for collection in self._find_collections(home):
                     yield collection
 
@@ -294,7 +294,6 @@ class DavStorage(Storage):
         if not href:
             raise ValueError(href)
         x = utils.urlparse.urljoin(self.session.url, href)
-        assert x.startswith(self.session.url)
         return utils.compat.urlunquote_plus(utils.urlparse.urlsplit(x).path)
 
     def _get_href(self, item):
@@ -543,7 +542,13 @@ class CaldavStorage(DavStorage):
         for caldavfilter in caldavfilters:
             xml = data.format(caldavfilter=caldavfilter)
             for href, etag in self._dav_query(xml):
-                assert href not in hrefs
+                if href in hrefs:
+                    # Can't do stronger assertion here, see
+                    # https://github.com/untitaker/vdirsyncer/issues/88
+                    dav_logger.warning('Skipping identical href: {}'
+                                       .format(href))
+                    continue
+
                 hrefs.add(href)
                 yield href, etag
 
@@ -611,6 +616,11 @@ class CarddavStorage(DavStorage):
             href = self._normalize_href(element.find('{DAV:}href').text)
             etag = prop.find('{DAV:}getetag').text
 
-            if href not in hrefs:
-                hrefs.add(href)
-                yield self._normalize_href(href), etag
+            if href in hrefs:
+                # Can't do stronger assertion here, see
+                # https://github.com/untitaker/vdirsyncer/issues/88
+                dav_logger.warning('Skipping identical href: {}'.format(href))
+                continue
+
+            hrefs.add(href)
+            yield self._normalize_href(href), etag
