@@ -12,7 +12,7 @@ import sys
 PY2 = sys.version_info[0] == 2
 
 
-if PY2:
+if PY2:  # pragma: no cover
     import urlparse
     from urllib import \
          quote_plus as urlquote_plus, \
@@ -20,12 +20,10 @@ if PY2:
     text_type = unicode  # flake8: noqa
     iteritems = lambda x: x.iteritems()
     itervalues = lambda x: x.itervalues()
-    get_raw_input = raw_input
-else:
+else:  # pragma: no cover
     import urllib.parse as urlparse
     urlquote_plus = urlparse.quote_plus
     urlunquote_plus = urlparse.unquote_plus
     text_type = str
     iteritems = lambda x: x.items()
     itervalues = lambda x: x.values()
-    get_raw_input = input
